@@ -65,7 +65,7 @@
 var $t$;
 //var c$;
 Clazz_declarePackage ("JS");
-Clazz_load (null, "JS.MathExt", ["java.lang.Float", "$.Number", "java.util.Date", "$.Hashtable", "$.Random", "JU.AU", "$.BS", "$.CU", "$.Lst", "$.M4", "$.Measure", "$.P3", "$.P4", "$.PT", "$.Quat", "$.SB", "$.V3", "J.api.Interface", "J.atomdata.RadiusData", "J.bspt.Bspt", "J.c.VDW", "J.i18n.GT", "JM.BondSet", "JS.SV", "$.ScriptParam", "$.T", "JU.BSUtil", "$.Escape", "$.JmolMolecule", "$.Logger", "$.Parser", "$.Point3fi", "$.SimpleUnitCell", "JV.FileManager", "$.JC"], function () {
+Clazz_load (null, "JS.MathExt", ["java.lang.Float", "$.Number", "java.util.Date", "$.Hashtable", "$.Random", "JU.AU", "$.BS", "$.CU", "$.Lst", "$.M4", "$.Measure", "$.OC", "$.P3", "$.P4", "$.PT", "$.Quat", "$.Rdr", "$.SB", "$.V3", "J.api.Interface", "J.atomdata.RadiusData", "J.bspt.PointIterator", "J.c.VDW", "J.i18n.GT", "JM.BondSet", "JS.SV", "$.ScriptParam", "$.T", "JU.BSUtil", "$.Escape", "$.JmolMolecule", "$.Logger", "$.Parser", "$.Point3fi", "$.SimpleUnitCell", "JV.FileManager", "$.JC", "$.Viewer"], function () {
 c$ = Clazz_decorateAsClass (function () {
 this.vwr = null;
 this.e = null;
@@ -147,6 +147,7 @@ return this.evaluateFormat (mp, op.intValue, args, tok == 1825200146);
 case 134320141:
 return this.evaluateUserFunction (mp, op.value, args, op.intValue, op.tok == 268435665);
 case 1275068449:
+case 1275068437:
 case 1275082245:
 case 1275072526:
 return this.evaluateGetProperty (mp, args, tok, op.tok == 268435665);
@@ -154,7 +155,7 @@ case 136314895:
 return this.evaluateHelix (mp, args);
 case 134219265:
 case 134217750:
-case 134219266:
+case 134217763:
 return this.evaluatePlane (mp, args, tok);
 case 134218759:
 case 134238732:
@@ -281,6 +282,8 @@ ucnew =  new Array (4);
 for (var i = 0; i < 4; i++) ucnew[i] =  new JU.P3 ();
 
 JU.SimpleUnitCell.setOabc (s, null, ucnew);
+} else if (s.indexOf (",") >= 0) {
+return mp.addXObj (this.vwr.getV0abc (s));
 }break;
 }
 var u = null;
@@ -333,8 +336,11 @@ var toPrimitive = "primitive".equalsIgnoreCase (op);
 if (toPrimitive || "conventional".equalsIgnoreCase (op)) {
 var stype = (++ptParam > lastParam ? "" : args[ptParam].asString ().toUpperCase ());
 if (stype.equals ("BCC")) stype = "I";
- else if (stype.length == 0) stype = this.vwr.getSymTemp ().getSymmetryInfoAtom (this.vwr.ms, iatom, null, 0, null, null, null, 1073741994, 0, -1);
-if (stype == null || stype.length == 0 || !(u == null ? this.vwr.getSymTemp () : u).toFromPrimitive (toPrimitive, stype.charAt (0), ucnew)) return false;
+ else if (stype.length == 0) stype = this.vwr.getSymmetryInfo (iatom, null, 0, null, null, 1073741994, null, 0, -1, 0);
+if (stype == null || stype.length == 0) return false;
+if (u == null) u = this.vwr.getSymTemp ();
+var m3 = this.vwr.getModelForAtomIndex (iatom).auxiliaryInfo.get ("primitiveToCrystal");
+if (!u.toFromPrimitive (toPrimitive, stype.charAt (0), ucnew, m3)) return false;
 } else if ("reciprocal".equalsIgnoreCase (op)) {
 ucnew = JU.SimpleUnitCell.getReciprocal (ucnew, null, scale);
 scale = 1;
@@ -801,53 +807,72 @@ a.cross (a, mp.ptValue (x2, null));
 return mp.addXPt (a);
 }var pt2 = (x2.tok == 7 ? null : mp.ptValue (x2, null));
 var plane2 = mp.planeValue (x2);
+var f = NaN;
+try {
 if (isDist) {
 var minMax = (op == -2147483648 ? 0 : op & 480);
 var isMinMax = (minMax == 32 || minMax == 64);
 var isAll = minMax == 480;
 switch (x1.tok) {
+case 7:
 case 10:
-var bs = x1.value;
-var bs2 = null;
+var isAtomSet1 = (x1.tok == 10);
+var isAtomSet2 = (x2.tok == 10);
+var isPoint2 = (x2.tok == 8);
+var bs1 = (isAtomSet1 ? x1.value : null);
+var bs2 = (isAtomSet2 ? x2.value : null);
+var list1 = (isAtomSet1 ? null : x1.getList ());
+var list2 = (isAtomSet2 ? null : x2.getList ());
 var returnAtom = (isMinMax && x3 != null && x3.asBoolean ());
 switch (x2.tok) {
 case 10:
-bs2 = (x2.tok == 10 ? x2.value : null);
+case 7:
 case 8:
 var atoms = this.vwr.ms.at;
 if (returnAtom) {
 var dMinMax = NaN;
 var iMinMax = 2147483647;
-for (var i = bs.nextSetBit (0); i >= 0; i = bs.nextSetBit (i + 1)) {
-var d = (bs2 == null ? atoms[i].distanceSquared (pt2) : (this.e.getBitsetProperty (bs2, op, atoms[i], plane2, x1.value, null, false, x1.index, false)).floatValue ());
+if (isAtomSet1) {
+for (var i = bs1.nextSetBit (0); i >= 0; i = bs1.nextSetBit (i + 1)) {
+var d = (isPoint2 ? atoms[i].distanceSquared (pt2) : (this.e.getBitsetProperty (bs2, list2, op, atoms[i], plane2, x1.value, null, false, x1.index, false)).floatValue ());
 if (minMax == 32 ? d >= dMinMax : d <= dMinMax) continue;
 dMinMax = d;
 iMinMax = i;
 }
 return mp.addXBs (iMinMax == 2147483647 ?  new JU.BS () : JU.BSUtil.newAndSetBit (iMinMax));
+}for (var i = list1.size (); --i >= 0; ) {
+var pt = JS.SV.ptValue (list1.get (i));
+var d = (isPoint2 ? pt.distanceSquared (pt2) : (this.e.getBitsetProperty (bs2, list2, op, pt, plane2, x1.value, null, false, 2147483647, false)).floatValue ());
+if (minMax == 32 ? d >= dMinMax : d <= dMinMax) continue;
+dMinMax = d;
+iMinMax = i;
+}
+return mp.addXInt (iMinMax);
 }if (isAll) {
 if (bs2 == null) {
-var data =  Clazz_newFloatArray (bs.cardinality (), 0);
-for (var p = 0, i = bs.nextSetBit (0); i >= 0; i = bs.nextSetBit (i + 1), p++) data[p] = atoms[i].distance (pt2);
+var data =  Clazz_newFloatArray (bs1.cardinality (), 0);
+for (var p = 0, i = bs1.nextSetBit (0); i >= 0; i = bs1.nextSetBit (i + 1), p++) data[p] = atoms[i].distance (pt2);
 
 return mp.addXAF (data);
-}var data2 =  Clazz_newFloatArray (bs.cardinality (), bs2.cardinality (), 0);
-for (var p = 0, i = bs.nextSetBit (0); i >= 0; i = bs.nextSetBit (i + 1), p++) for (var q = 0, j = bs2.nextSetBit (0); j >= 0; j = bs2.nextSetBit (j + 1), q++) data2[p][q] = atoms[i].distance (atoms[j]);
+}var data2 =  Clazz_newFloatArray (bs1.cardinality (), bs2.cardinality (), 0);
+for (var p = 0, i = bs1.nextSetBit (0); i >= 0; i = bs1.nextSetBit (i + 1), p++) for (var q = 0, j = bs2.nextSetBit (0); j >= 0; j = bs2.nextSetBit (j + 1), q++) data2[p][q] = atoms[i].distance (atoms[j]);
 
 
 return mp.addXAFF (data2);
 }if (isMinMax) {
-var data =  Clazz_newFloatArray (bs.cardinality (), 0);
-for (var i = bs.nextSetBit (0), p = 0; i >= 0; i = bs.nextSetBit (i + 1)) data[p++] = (this.e.getBitsetProperty (bs2, op, atoms[i], plane2, x1.value, null, false, x1.index, false)).floatValue ();
+var data =  Clazz_newFloatArray (isAtomSet1 ? bs1.cardinality () : list1.size (), 0);
+if (isAtomSet1) {
+for (var i = bs1.nextSetBit (0), p = 0; i >= 0; i = bs1.nextSetBit (i + 1)) data[p++] = (this.e.getBitsetProperty (bs2, list2, op, atoms[i], plane2, x1.value, null, false, x1.index, false)).floatValue ();
 
 return mp.addXAF (data);
-}return mp.addXObj (this.e.getBitsetProperty (bs, op, pt2, plane2, x1.value, null, false, x1.index, false));
+}for (var i = data.length; --i >= 0; ) data[i] = (this.e.getBitsetProperty (bs2, list2, op, JS.SV.ptValue (list1.get (i)), plane2, null, null, false, 2147483647, false)).floatValue ();
+
+return mp.addXAF (data);
+}return mp.addXObj (this.e.getBitsetProperty (bs1, list1, op, pt2, plane2, x1.value, null, false, x1.index, false));
 }
 }
 }var pt1 = mp.ptValue (x1, null);
 var plane1 = mp.planeValue (x1);
-var f = NaN;
-try {
 if (isDist) {
 if (plane2 != null && x3 != null) f = JU.Measure.directedDistanceToPlane (pt1, plane2, JS.SV.ptValue (x3));
  else f = (plane1 == null ? (plane2 == null ? pt2.distance (pt1) : JU.Measure.distanceToPlane (plane2, pt1)) : JU.Measure.distanceToPlane (plane1, pt2));
@@ -900,7 +925,7 @@ return false;
 Clazz_defineMethod (c$, "getHelixData", 
  function (bs, tokType) {
 var iAtom = bs.nextSetBit (0);
-return (iAtom < 0 ? "null" : this.vwr.ms.at[iAtom].group.getHelixData (tokType, this.vwr.getQuaternionFrame (), this.vwr.getInt (553648146)));
+return (iAtom < 0 ? "null" : this.vwr.ms.at[iAtom].group.getHelixData (tokType, this.vwr.getQuaternionFrame (), this.vwr.getInt (553648145)));
 }, "JU.BS,~N");
 Clazz_defineMethod (c$, "evaluateFind", 
  function (mp, args) {
@@ -961,7 +986,7 @@ case 3:
 asMap = JS.SV.bValue (args[2]);
 break;
 }
-var justOne = (!asMap && (!allMappings || !isSMARTS));
+var justOne = (!asMap && (!allMappings || !isSMARTS && !pattern.equals ("chirality")));
 try {
 ret = this.e.getSmilesExt ().getSmilesMatches (pattern, smiles, null, null, isSMARTS ? 2 : 1, !asMap, !allMappings);
 } catch (ex) {
@@ -997,8 +1022,18 @@ ret = map;
 ret = this.vwr.ms.generateCrystalClass (bs.nextSetBit (0), (args.length != 2 ? null : argLast.tok == 10 ? this.vwr.ms.getAtomSetCenter (argLast.value) : JS.SV.ptValue (argLast)));
 } else {
 var smilesFlags = (isSmiles ? (flags.indexOf ("OPEN") >= 0 ? 5 : 1) : 2) | (isON && sFind.length == 0 ? 22020096 : 0);
+if (flags.indexOf ("/MOLECULE/") >= 0) {
+var mols = this.vwr.ms.getMolecules ();
+var molList =  new JU.Lst ();
+for (var i = 0; i < mols.length; i++) {
+if (mols[i].atomList.intersects (bs)) {
+var bsRet = this.e.getSmilesExt ().getSmilesMatches (sFind, null, mols[i].atomList, bsMatch3D, smilesFlags, !isON, false);
+if (!bsRet.isEmpty ()) molList.addLast (bsRet);
+}}
+ret = molList;
+} else {
 ret = this.e.getSmilesExt ().getSmilesMatches (sFind, null, bs, bsMatch3D, smilesFlags, !isON, false);
-}break;
+}}break;
 }
 if (ret == null) this.e.invArg ();
 return mp.addXObj (ret);
@@ -1083,19 +1118,21 @@ return mp.addXStr (this.vwr.getJBR ().toStdAmino3 (JS.SV.sValue (x1)));
 }, "JS.ScriptMathProcessor,~A");
 Clazz_defineMethod (c$, "evaluateGetProperty", 
  function (mp, args, tok0, isAtomProperty) {
+var nargs = args.length;
 var isSelect = (isAtomProperty && tok0 == 1275082245);
+var isPivot = (isAtomProperty && tok0 == 1275068437 && nargs > 0);
 var isAuxiliary = (tok0 == 1275068449);
 var pt = 0;
-var tok = (args.length == 0 ? 0 : args[0].tok);
-if (args.length == 2 && (tok == 7 || tok == 6 || tok == 14)) {
+var tok = (nargs == 0 ? 0 : args[0].tok);
+if (nargs == 2 && (tok == 7 || tok == 6 || tok == 14)) {
 return mp.addXObj (this.vwr.extractProperty (args[0].value, args[1].value.toString (), -1));
-}var bsSelect = (isAtomProperty && args.length == 1 && args[0].tok == 10 ? args[0].value : null);
-var pname = (bsSelect == null && args.length > 0 ? JS.SV.sValue (args[pt++]) : "");
+}var bsSelect = (isAtomProperty && nargs == 1 && args[0].tok == 10 ? args[0].value : null);
+var pname = (bsSelect == null && nargs > 0 ? JS.SV.sValue (args[pt++]) : "");
 var propertyName = pname;
 var lc = propertyName.toLowerCase ();
 if (!isSelect && lc.indexOf ("[select ") < 0) propertyName = lc;
 var isJSON = false;
-if (propertyName.equals ("json") && args.length > pt) {
+if (propertyName.equals ("json") && nargs > pt) {
 isJSON = true;
 propertyName = JS.SV.sValue (args[pt++]);
 }var x = null;
@@ -1125,7 +1162,29 @@ data[1] = Integer.$valueOf (-1);
 this.vwr.shm.getShapePropertyData (shapeID, pname.intern (), data);
 }}return (data[1] == null ? mp.addXStr ("") : mp.addXObj (data[1]));
 case 7:
-if (bsSelect != null) {
+if (isPivot) {
+var lstx = x.getList ();
+var map =  new java.util.Hashtable ();
+var sep = (nargs > 1 ? JS.SV.sValue (args[nargs - 1]) : null);
+if (sep != null) nargs--;
+var keys =  new Array (nargs);
+for (var i = 0; i < nargs; i++) keys[i] = JS.SV.sValue (args[i]);
+
+for (var i = 0, n = lstx.size (); i < n; i++) {
+var sv = lstx.get (i);
+if (sv.tok != 6) continue;
+var mapi = sv.getMap ();
+var key = "";
+for (var j = 0; j < nargs; j++) {
+var obj = mapi.get (keys[j]);
+key += (j == 0 ? "" : sep) + JS.SV.sValue (obj);
+}
+var vlist = map.get (key);
+if (vlist == null) map.put (key, vlist = JS.SV.newV (7,  new JU.Lst ()));
+vlist.getList ().addLast (sv);
+}
+return mp.addXMap (map);
+}if (bsSelect != null) {
 var l0 = x.getList ();
 var lst =  new JU.Lst ();
 for (var i = bsSelect.nextSetBit (0); i >= 0; i = bsSelect.nextSetBit (i + 1)) lst.addLast (l0.get (i));
@@ -1135,19 +1194,19 @@ return mp.addXList (lst);
 if (isSelect) propertyName = "[SELECT " + propertyName + "]";
 return mp.addXObj (this.vwr.extractProperty (x, propertyName, -1));
 }
-if (!lc.startsWith ("bondinfo") && !lc.startsWith ("atominfo")) propertyName = "atomInfo." + propertyName;
+if (!lc.startsWith ("bondinfo") && !lc.startsWith ("atominfo") && !lc.startsWith ("modelkitinfo")) propertyName = "atomInfo." + propertyName;
 }var propertyValue = "";
-if (propertyName.equalsIgnoreCase ("fileContents") && args.length > 2) {
+if (propertyName.equalsIgnoreCase ("fileContents") && nargs >= 2) {
 var s = JS.SV.sValue (args[1]);
-for (var i = 2; i < args.length; i++) s += "|" + JS.SV.sValue (args[i]);
+for (var i = 2; i < nargs; i++) s += "|" + JS.SV.sValue (args[i]);
 
 propertyValue = s;
-pt = args.length;
-} else if (args.length > pt) {
+pt = nargs;
+} else if (nargs > pt) {
 switch (args[pt].tok) {
 case 10:
 propertyValue = args[pt++].value;
-if (propertyName.equalsIgnoreCase ("bondInfo") && args.length > pt && args[pt].tok == 10) propertyValue =  Clazz_newArray (-1, [propertyValue, args[pt].value]);
+if (propertyName.equalsIgnoreCase ("bondInfo") && nargs > pt && args[pt].tok == 10) propertyValue =  Clazz_newArray (-1, [propertyValue, args[pt].value]);
 break;
 case 6:
 case 4:
@@ -1162,7 +1221,7 @@ propertyValue = bs;
 }if (isAuxiliary && !isAtomProperty) propertyName = "auxiliaryInfo.models." + propertyName;
 propertyName = JU.PT.rep (propertyName, ".[", "[");
 var property = this.vwr.getProperty (null, propertyName, propertyValue);
-if (pt < args.length) property = this.vwr.extractProperty (property, args, pt);
+if (pt < nargs) property = this.vwr.extractProperty (property, args, pt);
 return mp.addXObj (isJSON ? JS.SV.safeJSON ("value", property) : JS.SV.isVariableType (property) ? property : JU.Escape.toReadable (propertyName, property));
 }, "JS.ScriptMathProcessor,~A,~N,~B");
 Clazz_defineMethod (c$, "evaluateFormat", 
@@ -1418,13 +1477,21 @@ var asBytes = (args.length > 1 && args[1].tok == 1073742335);
 var async = (this.vwr.async || args.length > 2 && args[args.length - 1].tok == 1073742335);
 var nBytesMax = (args.length > 1 && args[1].tok == 2 ? args[1].asInt () : -1);
 var asJSON = (args.length > 1 && args[1].asString ().equalsIgnoreCase ("JSON"));
-if (asBytes) return mp.addXMap (this.vwr.fm.getFileAsMap (file));
+if (asBytes) return mp.addXMap (this.vwr.fm.getFileAsMap (file, null));
 var isQues = file.startsWith ("?");
-if (this.vwr.isJS && (isQues || async)) {
+if (JV.Viewer.isJS && (isQues || async)) {
 if (isFile && isQues) return mp.addXStr ("");
 file = this.e.loadFileAsync ("load()_", file, mp.oPt, true);
 }var str = isFile ? this.vwr.fm.getFilePath (file, false, false) : this.vwr.getFileAsString4 (file, nBytesMax, false, false, true, "script");
+try {
 return (asJSON ? mp.addXObj (this.vwr.parseJSON (str)) : mp.addXStr (str));
+} catch (e) {
+if (Clazz_exceptionOf (e, Exception)) {
+return false;
+} else {
+throw e;
+}
+}
 }, "JS.ScriptMathProcessor,~A,~B");
 Clazz_defineMethod (c$, "evaluateMath", 
  function (mp, args, tok) {
@@ -1558,7 +1625,7 @@ return mp.addXList (this.vwr.ms.getModulationList (bs, (type + "D").toUpperCase
 }, "JS.ScriptMathProcessor,~A");
 Clazz_defineMethod (c$, "evaluatePlane", 
  function (mp, args, tok) {
-if (tok == 134219265 && args.length != 3 || tok == 134219266 && args.length != 2 && args.length != 3 || args.length == 0 || args.length > 4) return false;
+if (tok == 134219265 && args.length != 3 || tok == 134217763 && args.length != 2 && args.length != 3 && args.length != 4 || args.length == 0 || args.length > 4) return false;
 var pt1;
 var pt2;
 var pt3;
@@ -1576,7 +1643,7 @@ return mp.addXPt4 (JU.Measure.getPlaneThroughPoints (pts.get (0), pts.get (1), p
 if (Clazz_instanceOf (pt, JU.P4)) return mp.addXPt4 (pt);
 return mp.addXStr ("" + pt);
 case 2:
-if (tok == 134219266) {
+if (tok == 134217763) {
 if (args[1].tok != 9) return false;
 pt3 =  new JU.P3 ();
 norm =  new JU.V3 ();
@@ -1594,7 +1661,7 @@ case 4:
 switch (tok) {
 case 134219265:
 return mp.addXPt4 (this.e.getHklPlane (JU.P3.new3 (JS.SV.fValue (args[0]), JS.SV.fValue (args[1]), JS.SV.fValue (args[2]))));
-case 134219266:
+case 134217763:
 pt1 = mp.ptValue (args[0], null);
 pt2 = mp.ptValue (args[1], null);
 if (pt1 == null || pt2 == null) return mp.addXStr ("");
@@ -1610,8 +1677,24 @@ return mp.addXPt (pt1);
 }pt3 = mp.ptValue (args[2], null);
 if (pt3 == null) return mp.addXStr ("");
 var v =  new JU.V3 ();
+pt3 = JU.P3.newP (pt3);
+if (args.length == 3) {
 JU.Measure.projectOntoAxis (pt3, pt1, vLine, v);
 return mp.addXPt (pt3);
+}var r = JS.SV.fValue (args[3]);
+var ptCenter = JU.P3.newP (pt3);
+JU.Measure.projectOntoAxis (pt3, pt1, vLine, v);
+var d = ptCenter.distance (pt3);
+var l =  new JU.Lst ();
+if (d == r) {
+l.addLast (pt3);
+} else if (d < r) {
+d = Math.sqrt (r * r - d * d);
+v.scaleAdd2 (d, vLine, pt3);
+l.addLast (JU.P3.newP (v));
+v.scaleAdd2 (-d, vLine, pt3);
+l.addLast (JU.P3.newP (v));
+}return mp.addXList (l);
 }
 switch (args[0].tok) {
 case 2:
@@ -1959,7 +2042,7 @@ var s = JS.SV.sValue (args[0]);
 var sb =  new JU.SB ();
 switch (tok) {
 case 134218759:
-return (args.length == 2 ? s.equalsIgnoreCase ("JSON") && mp.addXObj (this.vwr.parseJSON (JS.SV.sValue (args[1]))) : mp.addXObj (this.vwr.evaluateExpressionAsVariable (s)));
+return (args.length == 2 ? s.equalsIgnoreCase ("JSON") && mp.addXObj (this.vwr.parseJSONMap (JS.SV.sValue (args[1]))) : mp.addXObj (this.vwr.evaluateExpressionAsVariable (s)));
 case 134222850:
 var appID = (args.length == 2 ? JS.SV.sValue (args[1]) : ".");
 if (!appID.equals (".")) sb.append (this.vwr.jsEval (appID + "\1" + s));
@@ -2094,7 +2177,7 @@ if (bsAtoms == null && this.vwr.ms.mc == 1) bsAtoms = this.vwr.getModelUndeleted
 var narg = args.length;
 if (narg == 0) {
 if (bsAtoms.isEmpty ()) return false;
-var ops = JU.PT.split (JU.PT.trim (this.vwr.getSymTemp ().getSpaceGroupInfo (this.vwr.ms, null, this.vwr.ms.at[bsAtoms.nextSetBit (0)].mi).get ("symmetryInfo"), "\n"), "\n");
+var ops = JU.PT.split (JU.PT.trim (this.vwr.getSymTemp ().getSpaceGroupInfo (this.vwr.ms, null, this.vwr.ms.at[bsAtoms.nextSetBit (0)].mi, false).get ("symmetryInfo"), "\n"), "\n");
 var lst =  new JU.Lst ();
 for (var i = 0, n = ops.length; i < n; i++) lst.addLast (JU.PT.split (ops[i], "\t"));
 
@@ -2123,11 +2206,11 @@ if (apt + 1 < narg && args[apt + 1].tok == 10) (bsAtoms == null ? (bsAtoms =  ne
 var pt2 = null;
 if ((pt1 = (narg > apt ? mp.ptValue (args[apt], bsAtoms) : null)) != null) apt++;
 if ((pt2 = (narg > apt ? mp.ptValue (args[apt], bsAtoms) : null)) != null) apt++;
-var nth = (pt2 != null && args.length > apt && iOp == -2147483648 && args[apt].tok == 2 ? args[apt].intValue : 0);
-if (nth > 0) apt++;
+var nth = (pt2 != null && args.length > apt && iOp == -2147483648 && args[apt].tok == 2 ? args[apt].intValue : -1);
+if (nth >= 0) apt++;
 if (iOp == -2147483648) iOp = 0;
 var desc = (narg == apt ? (pt2 != null ? "all" : pt1 != null ? "point" : "matrix") : JS.SV.sValue (args[apt++]).toLowerCase ());
-return (bsAtoms != null && !bsAtoms.isEmpty () && apt == args.length && mp.addXObj (this.vwr.getSymTemp ().getSymmetryInfoAtom (this.vwr.ms, bsAtoms.nextSetBit (0), xyz, iOp, pt1, pt2, desc, 0, 0, nth)));
+return (bsAtoms != null && !bsAtoms.isEmpty () && apt == args.length && mp.addXObj (this.vwr.getSymmetryInfo (bsAtoms.nextSetBit (0), xyz, iOp, pt1, pt2, 0, desc, 0, nth, 0)));
 }, "JS.ScriptMathProcessor,~A,~B");
 Clazz_defineMethod (c$, "evaluateTensor", 
  function (mp, args) {
@@ -2162,7 +2245,7 @@ for (var i = 0; i < args.length; i++) {
 params.addLast (args[i]);
 }
 if (isSelector) {
-return mp.addXObj (this.e.getBitsetProperty (x1.value, tok, null, null, x1.value,  Clazz_newArray (-1, [name, params]), false, x1.index, false));
+return mp.addXObj (this.e.getBitsetProperty (x1.value, null, tok, null, null, x1.value,  Clazz_newArray (-1, [name, params]), false, x1.index, false));
 }var $var = this.e.getUserFunctionResult (name, params, null);
 return ($var == null ? false : mp.addX ($var));
 }, "JS.ScriptMathProcessor,~S,~A,~N,~B");
@@ -2324,75 +2407,27 @@ return ((bs != null || pt != null) && mp.addXObj (this.vwr.ms.getUnitCellPointsW
 }if (pt != null || pts1 != null) {
 if (args[last].tok == 7) {
 var sv = args[last].getList ();
-var pts =  new JU.Lst ();
-var bspt =  new J.bspt.Bspt (3, 0);
-var iter;
-if (pt != null && Float.isNaN (pt.x)) {
-var p;
-var pt3 =  new Array (sv.size ());
-for (var i = pt3.length; --i >= 0; ) {
-var p3 = JS.SV.ptValue (sv.get (i));
-if (p3 == null) return false;
-p =  new JU.Point3fi ();
-p.setT (p3);
-p.i = i;
-pt3[i] = p;
-bspt.addTuple (p);
-}
-iter = bspt.allocateCubeIterator ();
-var bsp = JU.BSUtil.newBitSet2 (0, sv.size ());
-for (var i = pt3.length; --i >= 0; ) {
-iter.initialize (p = pt3[i], distance, false);
-var d2 = distance * distance;
-var n = 0;
-while (iter.hasMoreElements ()) {
-var pt2 = iter.nextElement ();
-if (bsp.get (pt2.i) && pt2.distanceSquared (p) <= d2 && (++n > 1)) bsp.clear (pt2.i);
-}
-}
-for (var i = bsp.nextSetBit (0); i >= 0; i = bsp.nextSetBit (i + 1)) pts.addLast (JU.P3.newP (pt3[i]));
+var ap3 =  new Array (sv.size ());
+for (var i = ap3.length; --i >= 0; ) ap3[i] = JS.SV.ptValue (sv.get (i));
 
-return mp.addXList (pts);
-}if (distance == 0) {
-if (pts1 == null) {
-var d2 = 3.4028235E38;
-var pt3 = null;
-for (var i = sv.size (); --i >= 0; ) {
-var pta = JS.SV.ptValue (sv.get (i));
-distance = pta.distanceSquared (pt);
-if (distance < d2) {
-pt3 = pta;
-d2 = distance;
-}}
-return (pt3 == null ? mp.addXStr ("") : mp.addXPt (pt3));
-}var ptsOut =  Clazz_newIntArray (pts1.size (), 0);
-for (var i = ptsOut.length; --i >= 0; ) {
-var d2 = 3.4028235E38;
-var imin = -1;
-pt = JS.SV.ptValue (pts1.get (i));
-for (var j = sv.size (); --j >= 0; ) {
-var pta = JS.SV.ptValue (sv.get (j));
-distance = pta.distanceSquared (pt);
-if (distance < d2) {
-imin = j;
-d2 = distance;
-}}
-ptsOut[i] = imin;
+var ap31 = null;
+if (pts1 != null) {
+ap31 =  new Array (pts1.size ());
+for (var i = ap31.length; --i >= 0; ) ap31[i] = JS.SV.ptValue (pts1.get (i));
+
+}var ret =  new Array (1);
+switch (J.bspt.PointIterator.withinDistPoints (distance, pt, ap3, ap31, ret)) {
+case 134217751:
+return mp.addXPt (ret[0]);
+case 1073742001:
+return mp.addXList (ret[0]);
+case 1275068418:
+return mp.addXAI (ret[0]);
+case 4:
+return mp.addXStr (ret[0]);
+default:
+return false;
 }
-return mp.addXAI (ptsOut);
-}for (var i = sv.size (); --i >= 0; ) {
-var ptp = JS.SV.ptValue (sv.get (i));
-bspt.addTuple (ptp);
-}
-iter = bspt.allocateCubeIterator ();
-iter.initialize (pt, distance, false);
-var d2 = distance * distance;
-while (iter.hasMoreElements ()) {
-var pt2 = iter.nextElement ();
-if (pt2.distanceSquared (pt) <= d2) pts.addLast (pt2);
-}
-iter.release ();
-return mp.addXList (pts);
 }return mp.addXBs (this.vwr.getAtomsNearPt (distance, pt));
 }if (tok == 1086324744) return mp.addXBs (this.vwr.ms.getSequenceBits (withinStr, bs,  new JU.BS ()));
 if (bs == null) bs =  new JU.BS ();
@@ -2409,8 +2444,19 @@ switch (args.length) {
 case 0:
 return false;
 case 1:
-if (!args[0].asString ().toUpperCase ().equals ("PNGJ")) break;
-return mp.addXMap (this.vwr.fm.getFileAsMap (null));
+var type = args[0].asString ().toUpperCase ();
+if (type.equals ("PNGJ")) {
+return mp.addXMap (this.vwr.fm.getFileAsMap (null, "PNGJ"));
+}if (JU.PT.isOneOf (type, ";ZIP;ZIPALL;JMOL;")) {
+var params =  new java.util.Hashtable ();
+var oc =  new JU.OC ();
+params.put ("outputChannel", oc);
+this.vwr.createZip (null, type, params);
+var bis = JU.Rdr.getBIS (oc.toByteArray ());
+params =  new java.util.Hashtable ();
+this.vwr.getJzt ().readFileAsMap (bis, params, null);
+return mp.addXMap (params);
+}break;
 }
 return mp.addXStr (this.e.getCmdExt ().dispatch (134221856, true, args));
 }, "JS.ScriptMathProcessor,~A");
@@ -2448,7 +2494,7 @@ var ndata = 0;
 var htPivot = null;
 while (true) {
 if (JU.AU.isAF (floatOrSVArray)) {
-if (tok == 1140850707) return "NaN";
+if (tok == 1275068437) return "NaN";
 data = floatOrSVArray;
 ndata = data.length;
 if (ndata == 0) break;
@@ -2456,7 +2502,7 @@ if (ndata == 0) break;
 sv = floatOrSVArray;
 ndata = sv.size ();
 if (ndata == 0) {
-if (tok != 1140850707) break;
+if (tok != 1275068437) break;
 } else {
 var sv0 = sv.get (0);
 if (sv0.tok == 8) return this.getMinMaxPoint (sv, tok);
@@ -2471,7 +2517,7 @@ break;
 var minMax;
 var isMin = false;
 switch (tok) {
-case 1140850707:
+case 1275068437:
 htPivot =  new java.util.Hashtable ();
 sum = minMax = 0;
 break;
@@ -2490,7 +2536,7 @@ sum = minMax = 0;
 var sum2 = 0;
 var n = 0;
 var isInt = true;
-var isPivot = (tok == 1140850707);
+var isPivot = (tok == 1275068437);
 for (var i = ndata; --i >= 0; ) {
 var svi = (sv == null ? JS.SV.vF : sv.get (i));
 var v = (isPivot ? 1 : data == null ? JS.SV.fValue (svi) : data[i]);
@@ -2504,7 +2550,7 @@ case 128:
 case 96:
 sum += v;
 break;
-case 1140850707:
+case 1275068437:
 isInt = new Boolean (isInt & (svi.tok == 2)).valueOf ();
 var key = svi.asString ();
 var ii = htPivot.get (key);
@@ -2519,7 +2565,7 @@ if (isInt) minMax = svi.intValue;
 }break;
 }
 }
-if (tok == 1140850707) {
+if (tok == 1275068437) {
 return htPivot;
 }if (n == 0) break;
 switch (tok) {
@@ -2651,7 +2697,7 @@ bs = JU.BSUtil.copy (bsA);
 bs.or (bsB);
 var nModels = this.vwr.ms.getModelBS (bs, false).cardinality ();
 withinAllModels = (nModels > 1);
-if (warnMultiModel && nModels > 1 && !this.e.tQuiet) this.e.showString (J.i18n.GT._ ("Note: More than one model is involved in this contact!"));
+if (warnMultiModel && nModels > 1 && !this.e.tQuiet) this.e.showString (J.i18n.GT.$ ("Note: More than one model is involved in this contact!"));
 }if (!bsA.equals (bsB)) {
 var setBfirst = (!localOnly || bsA.cardinality () < bsB.cardinality ());
 if (setBfirst) {
